from operator import itemgetter

from django import forms
from django.utils.translation import gettext_lazy as _

from .models import Property


class AddStockForm(forms.Form):
    part_number = forms.CharField(label=_("Component"))
    properties = forms.JSONField(initial={}, label=_("Properties"), required=False)
    quantity = forms.IntegerField(min_value=1, label=_("Quantity"))
    original_quantity = forms.IntegerField(min_value=1, label=_("Original quantity"))
    total_value = forms.DecimalField(
        min_value=0, max_digits=8, decimal_places=2, label=_("Total value")
    )
    location = forms.CharField(label=_("Location"), widget=forms.Textarea)
    owner = forms.CharField(label=_("Owner"))


AddStockFormSet = forms.formset_factory(AddStockForm, extra=0)


class SearchForm(forms.Form):
    PER_PAGE_CHOICES = [
        (10, "10"),
        (25, "25"),
        (50, "50"),
        (100, "100"),
    ]

    q = forms.CharField(label=_("Search"), required=False)
    per_page = forms.TypedChoiceField(
        choices=PER_PAGE_CHOICES,
        coerce=int,
        empty_value=25,
        label=_("Results per page"),
        widget=forms.Select(attrs={"form": "searchForm"}),
    )


class FilterForm(forms.Form):
    def __init__(self, properties, components, data=None, *args, **kwargs):
        super().__init__(data, *args, **kwargs)

        self.properties = properties

        for property in self.properties:
            choices = set()
            for value in property.value_set.filter(component__in=components):
                choices.add((getattr(value, property.value_field), str(value)))

            # FIXME: If two or more properties share the same slug, the last one will replace all the others
            self.fields[property.key] = forms.MultipleChoiceField(
                choices=list(sorted(choices, key=itemgetter(0))),
                label=property.name,
                required=False,
                widget=forms.SelectMultiple(attrs={"form": "searchForm"}),
            )

    def values(self):
        for property in self.properties:
            key = property.key
            if key not in self.cleaned_data or not self.cleaned_data[key]:
                continue

            yield property, self.cleaned_data[key]
