from django.db import models
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _


class Property(models.Model):
    class Type(models.TextChoices):
        QUANTITY = "quantity", _("Quantity")
        TEXT = "text", _("Text")

    name = models.CharField(max_length=50)
    filterable = models.BooleanField(
        help_text=_("Whether this property can be used to filter components")
    )
    type = models.CharField(choices=Type, max_length=8)
    searchable = models.BooleanField(
        help_text=_("Whether this property should be used in search")
    )
    unit = models.CharField(
        max_length=3, help_text=_("Symbol of this quantity's unit of measurement")
    )
    scale = models.SmallIntegerField(
        help_text=_("Exponent of the smallest possible value")
    )

    @property
    def key(self):
        return slugify(self.name).replace("-", "_")

    @property
    def value_field(self):
        match self.type:
            case Property.Type.QUANTITY:
                return "integer"
            case Property.Type.TEXT:
                return "text"
            case _:
                return None

    def __str__(self):
        return self.name + (
            f" [{self.unit}]" if self.type == Property.Type.QUANTITY else ""
        )


class Component(models.Model):
    part_number = models.CharField(max_length=30)
    properties = models.ManyToManyField(Property, through="Value")

    def __str__(self):
        return self.part_number


class Value(models.Model):
    class Meta:
        unique_together = ("component", "property")

    SI_PREFIXES = {
        -12: "p",
        -9: "n",
        -6: "µ",
        -3: "m",
        3: "k",
        6: "M",
        9: "G",
    }

    property = models.ForeignKey(Property, on_delete=models.CASCADE)
    component = models.ForeignKey(Component, on_delete=models.CASCADE)
    text = models.TextField()
    integer = models.BigIntegerField()

    def __str__(self):
        match self.property.type:
            case Property.Type.QUANTITY:
                s = str(self.integer)

                scale = self.property.scale
                if scale > 0:
                    s += "0" * scale
                elif scale < 0:
                    s = s.rjust(abs(scale) + 1, "0")
                    s = s[:scale] + "." + s[scale:]
                    s = s.rstrip("0").removesuffix(".")

                leading_zeroes = len(s.removeprefix("0.")) - len(
                    s.removeprefix("0.").lstrip("0")
                )
                if prefix := Value.SI_PREFIXES.get(-leading_zeroes // 3 * 3, ""):
                    s = s.removeprefix("0.").lstrip("0")
                    s = (
                        (s[: -leading_zeroes % 3] or "0")
                        + "."
                        + s[-leading_zeroes % 3 :]
                    )

                trailing_zeroes = len(s) - len(s.rstrip("0"))
                if not prefix and (
                    prefix := Value.SI_PREFIXES.get(trailing_zeroes // 3 * 3, "")
                ):
                    s = s.rstrip("0") + "0" * (trailing_zeroes % 3)

                return f"{s} {prefix}{self.property.unit}"
            case Property.Type.TEXT:
                return self.text
